SPECIAL_MSGTYPE_FONTCHAT = "fontchat";

-- Initialization


	function onInit()
	    if ChatManager then
	        ChatManager.registerSlashHandler("/fontchat", processfontChat);
		ChatManager.registerSlashHandler("/fc", processFontChat);
		ChatManager.registerSlashHandler("/lang", processLanguageChat);
		ChatManager.registerSpecialMsgHandler(SPECIAL_MSGTYPE_FONTCHAT, handleFontchat);
	-- send launch message
		local msg = {sender = "", font = "emotefont"};
    		msg.text = "4E Languages v2.1 Extension for FGII 4E ruleset. Copyright 2010 Smiteworks."
		ChatManager.registerLaunchMessage(msg);

	    end

		
		
	end


	function processFontChat(params)
		if User.isHost() then
			sendFontMessage(params,0)
		else
			local errormsg = {};
			errormsg.font = "systemfont";
			errormsg.text = "ERROR: Only the Host can use Font Chat";
			ChatManager.addMessage(errormsg);
			return;
		end
	end

	function processLanguageChat(params)
		sendFontMessage(params,1)
	end

	function sendFontMessage(params,translate)
		if ChatManager then
			local fontstring, textstring = string.match(params, "%s*(%S+)%s*(.*)");

			if not textstring then
				local errormsg = {};
				errormsg.font = "systemfont";
				errormsg.text = "Usage: /fontchat [font] [text]";
				ChatManager.addMessage(errormsg);
				return;
			end


			if not User.isHost() then
				if speaksLanguage(User.getCurrentIdentity(), fontstring) == 0 then
					local errormsg = {};
					errormsg.font = "systemfont";
					errormsg.text = "ERROR: You do not speak " .. fontstring;
					ChatManager.addMessage(errormsg);
					return
				end
			end

			local msg = {font = "chatfont"};	
			local langid = " is speaking " .. string.upper(string.sub(fontstring,1,1)) .. string.sub(fontstring,2) .. ":"
			if User.isHost() then
				msg.text = GmIdentityManager.getCurrent() .. langid

			else
				msg.text = User.getIdentityLabel() .. langid
			end
			if OptionsManager.isOption("PCHT", "on") then
				if User.isHost() then
					msg.icon = "portrait_gm_token";
				else
					msg.icon = "portrait_" .. User.getCurrentIdentity() .. "_chat";
				end
			end
			ChatManager.deliverMessage(msg);

			local fontmsg = {font = fontstring, mode="chat"};
			fontmsg.text = textstring;
			ChatManager.deliverMessage(fontmsg);

			if translate == 1 then
				fontmsg.font ="systemfont"
				fontmsg.text ="Translated: " .. fontmsg.text
				ChatManager.deliverMessage(fontmsg,"");

				ChatManager.sendSpecialMessage(SPECIAL_MSGTYPE_FONTCHAT, {fontstring, textstring});
			end
		end
		
	end


function trim (s)
	return (string.gsub(s, "^%s*(.-)%s*$", "%1"))
end

function localgetSafeChildValue(sourcenode, fieldname, defaultval)
	if sourcenode then
		local srcvalnode = sourcenode.getChild(fieldname);
		if srcvalnode then
			local returnval = srcvalnode.getValue();
			if returnval then
				return returnval;
			end
		end
	end
	
	return defaultval;
end

function speaksLanguage(target, language)
	local source = DB.findNode("charsheet." .. target .. ".languagelist")
	if source then
		for k, subnode in pairs(source.getChildren()) do
			local pclanguage = string.lower(localgetSafeChildValue(subnode, "name", ""))
			if trim(pclanguage) == trim(language) then
				return 1
			end
    		end

	end
	return 0
end

function handleFontchat(msguser, msgidentity, paramlist)
  if ChatManager then
	-- Get the parameters
	local language = paramlist[1];
	local langtext = paramlist[2];
	-- Cycle through identities to determine user for recipient
	local lower_msg = string.lower(language);
	local user = nil;
	local speakers = {}
	for k, v in ipairs(User.getAllActiveIdentities()) do
		if speaksLanguage(v,lower_msg) == 1 then
			local label = User.getIdentityLabel(v);
			local recpient_user = User.getIdentityOwner(v)
			speakers[recpient_user] = recpient_user 
		end
	end
	local fontmsg = {font = "systemfont", text = "Translated: " .. langtext, mode = "chat"};

	for k,target in pairs(speakers) do
		ChatManager.deliverMessage(fontmsg,target);
	end

  end
end

